@@ -1,4 +1,4 @@
-from contextlib import contextmanager, closing
+from contextlib import closing, contextmanager
 
 from docker import DockerClient
 
